
/*
 * Monto - a wm dockapp mounter
 *
 * Copyright (C) 2003 pasp <pasp@ll.pl>, sill <sill@ll.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "docklib.h"
#include "monto.h"

/******************************************************************************/

int get_mount_points(void)
{
int i;
int valid_mp;

    valid_mp = 0;

    for(i=0; i<MAX_MOUNT_POINTS; i++) {

        mpp[i].invalid = true;

        if(dcl_check_file(mpp[i].path)==F_DIRECTORY) {

            mpp[i].invalid = false;
            valid_mp++;

        }

        if(!strlen(mpp[i].name) && mpp[i].invalid == false) {

            fprintf(stderr, "ERROR: entry has no label... removed\n");
            mpp[i].invalid = true;
            valid_mp--;

        }

    }

    return valid_mp;
}

/******************************************************************************/

int mount_check(char *mp)
{
struct stat statbuf;
int retcode;
dev_t prvdev, dirdev;
char *parent_ptr, mnt_point[PATH_MAX];


    dcl_strcpy(mnt_point, mp, PATH_MAX);

    retcode = stat(mnt_point, &statbuf);

    if (retcode != -1) {

        dirdev = statbuf.st_dev;
        parent_ptr = strrchr(mnt_point, '/');

        if (parent_ptr && parent_ptr != mnt_point)
            *parent_ptr = 0;
        else
            dcl_strcpy(mnt_point, "/", PATH_MAX);

        retcode = stat(mnt_point, &statbuf);

        if (retcode != -1) {    
            prvdev = statbuf.st_dev;

            if (prvdev != dirdev)
                    retcode = 1;
            else
                    retcode = 0;
        }
    }

    return retcode;
}

/******************************************************************************/

void mount_action(int point)
{
char command[PATH_MAX];
int i, j;

    if(mpp[point].invalid == false) {

        for(i=j=0; i<point; i++)
            if(mpp[i].invalid == false) j++;

        dcl_draw_led(LED_X_POS, LED_Y_POS+j*15, SKYBLUE);
        dcl_redraw_window();

        for(i=0; i<MAX_MOUNT_POINTS; i++)
            mpp[i].status = mount_check(mpp[i].path);

        point %= MAX_MOUNT_POINTS;

        if(mpp[point].status == 1) {
            sprintf(command, "%s %s", umount_command, mpp[point].path);
            dcl_execute_command(command, 1);
            sprintf(command, "%s %s", eject_command, mpp[point].path);
            dcl_execute_command(command, 1);
        }

        if(mpp[point].status == 0) {
            sprintf(command, "%s %s", mount_command, mpp[point].path);
            dcl_execute_command(command, 1);
        }

        for(i=0; i<MAX_MOUNT_POINTS; i++) 
            mpp[i].status = mount_check(mpp[i].path);

    }

}

