
/*
 * Monto - a wm dockapp mounter
 *
 * Copyright (C) 2003 pasp <pasp@ll.pl>, sill <sill@ll.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <time.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>

#include <sys/wait.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>

#include "docklib.h"

#include "monto.h"
#include "io-mount.h"

#include "master.h"
#include "mask.h"

XEvent Event;
int update_period = 1, mount_points;

mount_point_s mpp[MAX_MOUNT_POINTS];
char mount_command[PATH_MAX], umount_command[PATH_MAX];
char eject_command[PATH_MAX], browse_command[PATH_MAX];
char temp[PATH_MAX];

unsigned char *masks[4] = {
    monto_mask1_bits,
    monto_mask2_bits,
    monto_mask3_bits,
    monto_mask4_bits
};

int mregs[4*4] = {  /* mouse regions */ 
   3,  3, 60, 15,
   3, 18, 60, 30,
   3, 33, 60, 45,
   3, 48, 60, 60
};

/*----------------------------------------------------------------------*/

int monto_routine(int argc, char **argv)
{
int i, j, mp;
int buttons_state = -1;
long int starttime, curtime, nexttime;

int xfd = 0;
fd_set inputs;
struct timeval timeout;

    mp = mount_points - 1;
                 
    if (mp > MOUNT_POINTS_PER_PAGE)
        mp = mount_points % MOUNT_POINTS_PER_PAGE; 

    timeout.tv_sec = 0;
    timeout.tv_usec = 150000;

    dcl_open_x_window(argc, argv, monto_master_xpm, masks[mp], MASK_WIDTH, MASK_HEIGHT);

    /* leds off and put titles */
    for(i=j=0; i<MOUNT_POINTS_PER_PAGE; i++) {

        if(mpp[i].invalid == false) {
            dcl_draw_led(LED_X_POS, LED_Y_POS+j*15, GRAY);
            dcl_draw_string(15, LED_Y_POS+j*15, mpp[i].name, FONT_NORMAL, 7);
            j++;
        }

    }

    /* add mouse regions */

    for(i=j=0; i<MOUNT_POINTS_PER_PAGE; i++) {

        if(mpp[i].invalid == false) {
            dcl_add_mouse_region(i, mregs[j*4+0], mregs[j*4+1], mregs[j*4+2], mregs[j*4+3]);
            j++;
        }
    }

    dcl_redraw_window();

    /* main routine */
    
    starttime = time(0);
    nexttime = starttime + update_period;

    while (1) {
        curtime = time(0);

        waitpid(0, NULL, WNOHANG);

        /*=======================================================================*/
        /* checks mounted devices */

        if (curtime >= nexttime) {
            nexttime += update_period;

            for(i=j=0; i<MOUNT_POINTS_PER_PAGE; i++) {

                if(mpp[i].invalid == false) {

                    if(mount_check(mpp[i].path)) {

                        dcl_draw_led(LED_X_POS, LED_Y_POS+j*15, GREEN);
						mpp[i].status = 1;

					} else {

                        dcl_draw_led(LED_X_POS, LED_Y_POS+j*15, GRAY);
						mpp[i].status = 0;

					}
                    j++;
                }
            }

        }

        /*=======================================================================*/

        xfd = ConnectionNumber(display);

        FD_ZERO(&inputs);
        FD_SET(xfd, &inputs);

        switch(select(FD_SETSIZE, &inputs, NULL, NULL, &timeout)) {

            case 0:
                timeout.tv_sec = 0;
                timeout.tv_usec = 150000;
                dcl_redraw_window();
                break;

            case -1:
                break;

            default:
                break;
        }


        while(FD_ISSET(xfd, &inputs) && XPending(display)) {

            XNextEvent(display, &Event);

            switch (Event.type) {

                case Expose:
                    dcl_redraw_window();
                    break;

                case DestroyNotify:
                    XCloseDisplay(display);
                    exit(0);
                    break;

                case ButtonPress:
                    buttons_state = dcl_check_mouse_region(Event.xbutton.x, Event.xbutton.y);
                    break;

                case ButtonRelease:
                    i = dcl_check_mouse_region(Event.xbutton.x, Event.xbutton.y);
                    if(i != -1) {

                        /* LMB is pressed ? */    
                        if(Event.xbutton.button == LMB)
                            mount_action(i);

                        /* RMB is pressed and device is mounted ? */    
                        if(Event.xbutton.button == RMB && mpp[i].status) {

                            snprintf(temp, PATH_MAX, browse_command, mpp[i].path);
                            dcl_execute_command(temp, 0);

                        }

                    }
                    break;
            }
        }

    }
}

/*----------------------------------------------------------------------*/

void monto_write_prefs(void) 
{
int i;
char tmpbuf[PATH_MAX+MAX_LINE_LEN];

    if (dcl_prefs_openfile (dcl_getfilename_config (".clay", "monto"), P_WRITE)) {
    
        for(i=0; i<MAX_MOUNT_POINTS; i++) {
            sprintf(tmpbuf, "mount_point_%d", i+1);
            dcl_prefs_put_string (tmpbuf, mpp[i].path);
            sprintf(tmpbuf, "mount_point_name_%d", i+1);
            dcl_prefs_put_string (tmpbuf, mpp[i].name);
            dcl_prefs_put_lf ();
        }

        dcl_prefs_put_string ("mount_command", mount_command);
        dcl_prefs_put_string ("umount_command", umount_command);
        dcl_prefs_put_string ("eject_command", eject_command);
        dcl_prefs_put_string ("browse_command", browse_command);

    }
        
    dcl_prefs_closefile ();
}

/*----------------------------------------------------------------------*/

void monto_read_prefs(void) 
{
int i;
char tmpbuf[PATH_MAX+MAX_LINE_LEN];

    if (dcl_prefs_openfile (dcl_getfilename_config(".clay", "monto"), P_READ)) {

        for(i=0; i<MAX_MOUNT_POINTS; i++) {
            sprintf(tmpbuf, "mount_point_%d", i+1);
            dcl_strcpy(mpp[i].path, dcl_prefs_get_string (tmpbuf), PATH_MAX);
            sprintf(tmpbuf, "mount_point_name_%d", i+1);
            dcl_strcpy(mpp[i].name, dcl_prefs_get_string (tmpbuf), MAX_LINE_LEN);
        }

        dcl_strcpy(mount_command, dcl_prefs_get_string ("mount_command"), PATH_MAX);
        dcl_strcpy(umount_command, dcl_prefs_get_string ("umount_command"), PATH_MAX);
        dcl_strcpy(eject_command, dcl_prefs_get_string ("eject_command"), PATH_MAX);
        dcl_strcpy(browse_command, dcl_prefs_get_string ("browse_command"), PATH_MAX);
        
        dcl_prefs_closefile ();

        if(!strlen(browse_command)) {
            dcl_strcpy(browse_command, "rox -d %s", PATH_MAX);
            monto_write_prefs ();
        }

    } else {

        for(i=0; i<MAX_MOUNT_POINTS; i++) {
            dcl_strcpy(mpp[i].path, "/mnt/floppy", PATH_MAX);
            dcl_strcpy(mpp[i].name, "floppy", MAX_LINE_LEN);
        }

        dcl_strcpy(mount_command, "mount", PATH_MAX);
        dcl_strcpy(umount_command, "umount", PATH_MAX);
        dcl_strcpy(eject_command, "eject", PATH_MAX);
        dcl_strcpy(browse_command, "rox -d %s", PATH_MAX);

        monto_write_prefs ();

    }
}

/*----------------------------------------------------------------------*/

int main(int argc, char **argv)
{
    monto_read_prefs(); 

    mount_points = get_mount_points();

    if(!mount_points) {

        printf("ERROR: Please define at least one valid mount point to run monto (edit ~/.clay/monto file)...\n");
        return false;
    }

    monto_routine(argc, argv);
    return 0;
}


